@@ -16,41 +16,34 @@
 
 """Formatters for Nested Diff."""
 
-import html
-
 import nested_diff
+import nested_diff.handlers
+
+from html import escape as escape_html
 
 
 class AbstractFormatter(object):
     """Base class for nested diff formatters."""
 
-    def __init__(self, indent='  ', line_separator='\n', **kwargs):
+    def __init__(
+        self,
+        handlers=None,
+        indent='  ',
+        line_separator='\n',
+        sort_keys=True,
+    ):
         """
         Initialize formatter.
 
+        :param handlers: iterable with type handlers.
         :param indent: prefix for each level of diff.
-        :line_separator: text lines delimiter.
-
-        Rest kwargs passed to nested_diff.Iterator as is.
+        :param line_separator: text lines delimiter.
+        :param sort_keys: sort keys for dict-like structures.
 
         """
-        self.iterator = nested_diff.Iterator(**kwargs)
         self.indent = indent
         self.line_separator = line_separator
-
-        self.unified_header_prefix = '@@ -'
-        self.unified_header_suffix = ' @@'
-
-        self.obj_prefix = {
-            dict: '{',
-            list: '[',
-            tuple: '(',
-        }
-        self.obj_suffix = {
-            dict: '}',
-            list: ']',
-            tuple: ')',
-        }
+        self.sort_keys = sort_keys
 
         self.key_line_prefix = {
             'A': '+ ',
@@ -67,7 +60,7 @@ class AbstractFormatter(object):
         self.val_line_prefix = self.key_line_prefix.copy()
         self.val_line_prefix['C'] = '# '
         self.val_line_prefix['E'] = '# '
-        self.val_line_prefix['I'] = '  '
+        self.val_line_prefix['H'] = '  '
         self.val_line_prefix['O'] = '- '
         self.val_line_prefix['N'] = '+ '
 
@@ -86,32 +79,34 @@ class AbstractFormatter(object):
             'U',
         )
 
-        self.__yielders = {}
+        self.type_prefix = {}
+        self.type_suffix = {}
 
-    def get_yielder(self, diff, depth=0):
-        """Return apropriate yielder for diff extention."""
-        try:
-            return self.__yielders[diff['E'].__class__](diff, depth=depth)
-        except KeyError:
-            raise NotImplementedError from None
+        self.handlers = {}
 
-    @staticmethod
-    def get_unified_diff_range(start, stop):
-        """Return unified diff lines range."""
-        length = stop - start
+        if handlers is None:
+            handlers = (
+                *nested_diff._TYPE_HANDLERS,
+                nested_diff.handlers.TextHandler(),
+            )
 
-        if length > 1:
-            return '{},{}'.format(start + 1, length)
-
-        return str(start + 1)
+        for handler in handlers:
+            self.set_handler(handler)
 
     def format(self, diff, **kwargs):  # noqa A003
-        """Return completely formatted diff as string."""
-        return ''.join(self.yield_diff(diff, **kwargs))
+        """Return formatted diff."""
+        return ''.join(self.generate_diff(diff, **kwargs))
 
-    def set_yielder(self, type_, method):
-        """Set yielder for diff extention."""
-        self.__yielders[type_] = method
+    def set_handler(self, handler):
+        """
+        Set handler.
+
+        :param handler: handlers.TypeHandler.
+
+        """
+        self.handlers[handler.handled_type] = handler
+        self.type_prefix[handler.handled_type] = handler.type_prefix
+        self.type_suffix[handler.handled_type] = handler.type_suffix
 
 
 class TextFormatter(AbstractFormatter):
@@ -127,54 +122,10 @@ class TextFormatter(AbstractFormatter):
 
         """
         super().__init__(*args, **kwargs)
-
         self.type_hints = type_hints
 
-        self.set_yielder(frozenset, self.yield_set_diff)
-        self.set_yielder(set, self.yield_set_diff)
-        self.set_yielder(str, self.yield_text_diff)
-
-    def yield_text_diff(self, diff, depth=0):
-        """Yield unified text diff."""
-        indent = self.indent * depth
-
-        for subdiff in diff['D']:
-            for tag in ('I', 'R', 'A', 'U'):
-                if tag in subdiff:
-                    yield self.val_line_prefix[tag]
-                    yield indent
-
-                    value = subdiff[tag]
-                    if tag == 'I':
-                        yield self.unified_header_prefix
-                        yield self.get_unified_diff_range(value[0], value[1])
-                        yield ' +'
-                        yield self.get_unified_diff_range(value[2], value[3])
-                        yield self.unified_header_suffix
-                    else:
-                        yield self.val_prefix[tag]
-                        yield self.format_string(value)
-                        yield self.val_suffix[tag]
-
-                    yield self.line_separator
-
-    def yield_set_diff(self, diff, depth=0):
-        """Yield set and frozenset diff."""
-        indent = self.indent * depth
-
-        for subdiff in diff['D']:
-            for tag in ('R', 'A', 'U'):
-                if tag in subdiff:
-                    yield self.val_line_prefix[tag]
-                    yield indent
-                    yield self.val_prefix[tag]
-                    yield self.format_value(subdiff[tag])
-                    yield self.val_suffix[tag]
-                    yield self.line_separator
-                    break
-
-    def yield_comment(self, diff, depth=0):
-        """Yield diff comment."""
+    def generate_comment(self, diff, depth=0):
+        """Generate diff comment."""
         try:
             comment = diff['C']
             tag = 'C'
@@ -188,50 +139,54 @@ class TextFormatter(AbstractFormatter):
             except KeyError:
                 return
 
+        yield from self.generate_string(comment, tag, depth)
+
+    def generate_diff(self, diff, depth=0, header='', footer=''):
+        """Generate formatted diff."""
+        yield header
+
+        yield from self.generate_comment(diff, depth=depth)
+
+        try:
+            handler = self.handlers[diff['E'].__class__]
+        except KeyError:
+            try:
+                handler = self.handlers[diff['D'].__class__]
+            except KeyError:
+                handler = nested_diff._DEFAULT_HANDLER
+
+        yield from handler.generate_formatted_diff(self, diff, depth)
+
+        yield footer
+
+    def generate_key(self, key, tag, diff_type, depth):
+        """Generate key line."""
+        yield self.key_line_prefix[tag]
+        yield self.indent * depth
+        yield self.key_prefix[tag]
+        yield self.type_prefix[diff_type]
+        yield self.format_key(key)
+        yield self.type_suffix[diff_type]
+        yield self.key_suffix[tag]
+        yield self.line_separator
+
+    def generate_string(self, value, tag, depth):
+        """Generate string line."""
         yield self.val_line_prefix[tag]
         yield self.indent * depth
         yield self.val_prefix[tag]
-        yield self.format_string(comment)
+        yield self.format_string(value)
         yield self.val_suffix[tag]
         yield self.line_separator
 
-    def yield_diff(self, diff, depth=0, header='', footer=''):
-        """Yield formatted diff."""
-        yield header
-
-        for diff, key, subdiff, depth in self.iterator.iterate(diff, depth):  # noqa B020
-            yield from self.yield_comment(diff, depth=depth)
-
-            # value
-            if 'E' in diff:
-                yield from self.get_yielder(diff, depth=depth)
-                continue
-
-            if subdiff is None:
-                for tag in self.tags:
-                    if tag in diff:
-                        yield self.val_line_prefix[tag]
-                        yield self.indent * depth
-                        yield self.val_prefix[tag]
-                        yield self.format_value(diff[tag])
-                        yield self.val_suffix[tag]
-                        yield self.line_separator
-                continue
-
-            # key
-            for tag in self.tags:
-                if tag in subdiff:
-                    yield self.key_line_prefix[tag]
-                    yield self.indent * depth
-                    yield self.key_prefix[tag]
-                    yield self.obj_prefix[diff['D'].__class__]
-                    yield self.format_key(key)
-                    yield self.obj_suffix[diff['D'].__class__]
-                    yield self.key_suffix[tag]
-                    yield self.line_separator
-                    break
-
-        yield footer
+    def generate_value(self, value, tag, depth):
+        """Generate value line."""
+        yield self.val_line_prefix[tag]
+        yield self.indent * depth
+        yield self.val_prefix[tag]
+        yield self.format_value(value)
+        yield self.val_suffix[tag]
+        yield self.line_separator
 
     @staticmethod
     def format_key(key):
@@ -269,9 +224,6 @@ class HtmlFormatter(TextFormatter):
 
         self.line_separator = '</div>' + self.line_separator
 
-        self.unified_header_prefix = '<span class="dif-kX0-0">@@ -'
-        self.unified_header_suffix = ' @@</span>'
-
         for key, val in self.key_line_prefix.items():
             self.key_line_prefix[key] = '<div>' + val
         for key, val in self.val_line_prefix.items():
@@ -283,9 +235,9 @@ class HtmlFormatter(TextFormatter):
 
         for key, val in self.val_prefix.items():
             self.val_prefix[key] = ('<span class="dif-v' + key + '">' +
-                                    html.escape(val))
+                                    escape_html(val))
         for key, val in self.val_suffix.items():
-            self.val_suffix[key] = html.escape(val) + '</span>'
+            self.val_suffix[key] = escape_html(val) + '</span>'
 
     @staticmethod
     def get_css():
@@ -298,36 +250,36 @@ class HtmlFormatter(TextFormatter):
             ' .dif-kO {color: #000}'
             ' .dif-kR {background-color: #fcc}'
             ' .dif-kU {color: #777}'
-            ' .dif-kX0-0 {color: #707}'
             ' .dif-vA {background-color: #dfd}'
             ' .dif-vC {color: #00b}'
             ' .dif-vE {color: #00b}'
+            ' .dif-vH {color: #707}'
             ' .dif-vN {background-color: #dfd}'
             ' .dif-vO {background-color: #fdd}'
             ' .dif-vR {background-color: #fdd}'
             ' .dif-vU {color: #777}'
         )
 
-    def yield_diff(self, diff, depth=0, header='', footer=''):
-        """Yield formatted diff parts."""
-        yield from super().yield_diff(
+    def format(self, diff, header='', footer='', **kwargs):  # noqa A003
+        """Return completely formatted diff as string."""
+        return ''.join(self.generate_diff(
             diff,
-            depth=depth,
-            header=header + '<div class="dif-body">',
-            footer='</div>' + footer,
-        )
+            header=header+'<div class="dif-body">',
+            footer='</div>'+footer,
+            **kwargs,
+        ))
 
     def format_key(self, key):
         """Return key/index representation."""
-        return html.escape(super().format_key(key))
+        return escape_html(super().format_key(key))
 
     def format_string(self, val):
         """Return string representation."""
-        return html.escape(super().format_string(val))
+        return escape_html(super().format_string(val))
 
     def format_value(self, val):
         """Return value representation."""
-        return html.escape(super().format_value(val))
+        return escape_html(super().format_value(val))
 
 
 class TermFormatter(TextFormatter):
@@ -350,7 +302,7 @@ class TermFormatter(TextFormatter):
         self.val_line_prefix['A'] = '\033[32m' + self.val_line_prefix['A']
         self.val_line_prefix['C'] = '\033[34m' + self.val_line_prefix['C']
         self.val_line_prefix['E'] = '\033[34m' + self.val_line_prefix['E']
-        self.val_line_prefix['I'] = '\033[35m' + self.val_line_prefix['I']
+        self.val_line_prefix['H'] = '\033[35m' + self.val_line_prefix['H']
         self.val_line_prefix['N'] = '\033[32m' + self.val_line_prefix['N']
         self.val_line_prefix['O'] = '\033[31m' + self.val_line_prefix['O']
         self.val_line_prefix['R'] = '\033[31m' + self.val_line_prefix['R']
